from pydantic import BaseModel, Field, RootModel
from typing import List, Optional, TypeVar


class ModelMixin:
    def __str__(self):
        return repr(self)


class Object(ModelMixin, BaseModel):
    id: str = Field(alias="_id")


class ObjectWithTimestamp(ModelMixin, BaseModel):
    id: str = Field(alias="_id")
    created_at: str
    updated_at: Optional[str] = None


class PageInfo(BaseModel):
    limit_per_page: int
    page: int
    total_page: int
    total_results: int


class TokenPageInfo(BaseModel):
    limit_per_page: int
    total_results: int
    page_expires_at: str
    next_page_token: Optional[str] = None
    prev_page_token: Optional[str] = None


T = TypeVar("T")


class RootModelList(RootModel[List[T]]):
    """
    See https://docs.pydantic.dev/latest/concepts/models/#rootmodel-and-custom-root-types
    """

    def __init__(self, _list: List[T]):
        super().__init__(root=_list)

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]

    def __len__(self):
        return len(self.root)



from typing import Union, List, Literal, Optional

from ..resource import APIResource
from .. import models
from ..util import remove_none_values


class Generate(APIResource):
    def summarize(
        self,
        video_id: str,
        type: Literal["summary", "chapter", "highlight"],
        *,
        prompt: Optional[str] = None,
        temperature: Optional[float] = None,
        **kwargs,
    ) -> models.GenerateSummarizeResult:
        json = {
            "video_id": video_id,
            "type": type,
            "prompt": prompt,
            "temperature": temperature,
        }
        res = self._post("summarize", json=remove_none_values(json), **kwargs)
        return models.GenerateSummarizeResult(**res)

    def text(
        self,
        video_id: str,
        prompt: str,
        *,
        temperature: Optional[float] = None,
        **kwargs,
    ) -> models.GenerateOpenEndedTextResult:
        json = {
            "video_id": video_id,
            "prompt": prompt,
            "temperature": temperature,
            "stream": False,
        }
        res = self._post("generate", json=json, **kwargs)
        return models.GenerateOpenEndedTextResult(**res)

    def text_stream(
        self,
        video_id: str,
        prompt: str,
        *,
        temperature: Optional[float] = None,
        **kwargs,
    ) -> models.GenerateOpenEndedTextStreamResult:
        json = {
            "video_id": video_id,
            "prompt": prompt,
            "temperature": temperature,
            "stream": True,
        }
        res = self._post("generate", json=json, stream=True, **kwargs)
        return models.GenerateOpenEndedTextStreamResult(stream=res)



from __future__ import annotations

from typing import List, Optional, TYPE_CHECKING, Dict, Any, Union, Literal
from pydantic import Field, PrivateAttr

from ._base import BaseModel, ModelMixin, RootModelList, TokenPageInfo

if TYPE_CHECKING:
    from ..resources import Search as SearchResource


class SearchPool(BaseModel):
    total_count: int
    total_duration: float
    index_id: str


class SearchData(BaseModel):
    score: float
    start: float
    end: float
    video_id: str
    confidence: str
    thumbnail_url: Optional[str] = None
    module_confidence: Optional[Dict[str, Any]] = None


class GroupByVideoSearchData(BaseModel):
    clips: Optional[RootModelList[SearchData]] = None
    id: str


class SearchResult(ModelMixin, BaseModel):
    _resource: SearchResource = PrivateAttr()
    pool: SearchPool = Field(alias="search_pool")
    data: RootModelList[Union[SearchData, GroupByVideoSearchData]]
    page_info: TokenPageInfo

    def __init__(self, resource: SearchResource, **data):
        super().__init__(**data)
        self._resource = resource

    def __iter__(self):
        return self

    def __next__(self) -> RootModelList[Union[SearchData, GroupByVideoSearchData]]:
        next_page_token = self.page_info.next_page_token
        if not next_page_token:
            raise StopIteration

        res = self._resource.by_page_token(next_page_token)
        self.page_info = res.page_info
        return res.data


class CombinedSearchResult(SearchResult):
    def __next__(self) -> RootModelList[SearchData]:
        next_page_token = self.page_info.next_page_token
        if not next_page_token:
            raise StopIteration

        res = self._resource.combined_by_page_token(next_page_token)
        self.page_info = res.page_info
        return res.data



import httpx
import json
from typing import List, Optional
from pydantic import PrivateAttr

from ._base import BaseModel, RootModelList


class GenerateOpenEndedTextResult(BaseModel):
    id: str
    data: str


class GenerateSummarizeChapterResult(BaseModel):
    chapter_number: int
    start: float
    end: float
    chapter_title: str
    chapter_summary: str


class GenerateSummarizeHighlightResult(BaseModel):
    start: float
    end: float
    highlight: str


class GenerateSummarizeResult(BaseModel):
    id: str
    summary: Optional[str] = None
    chapters: Optional[RootModelList[GenerateSummarizeChapterResult]] = None
    highlights: Optional[RootModelList[GenerateSummarizeHighlightResult]] = None


class GenerateOpenEndedTextStreamResult(BaseModel):
    _stream: httpx.Response = PrivateAttr()
    id: str = ""
    texts: List[str] = []
    aggregated_text: str = ""

    def __init__(self, stream: httpx.Response, **data):
        super().__init__(**data)
        self._stream = stream
        self.id = ""
        self.texts = []
        self.aggregated_text = ""

    def add_text(self, text: str):
        self.texts.append(text)
        self.aggregated_text += text

    def __iter__(self):
        buffer = ""
        with self._stream as response:
            for line in response.iter_lines():
                buffer += line
                while True:
                    try:
                        # Try to decode a JSON object from the buffer
                        event, index = json.JSONDecoder().raw_decode(buffer)
                        # Update the buffer to remove the decoded JSON object
                        buffer = buffer[index:].lstrip()
                    except json.JSONDecodeError:
                        break

                    if "code" in event:
                        # Handle non-stream error
                        raise Exception(event["message"])
                    elif event["event_type"] == "stream_start":
                        self.id = event["metadata"]["generation_id"]
                    elif event["event_type"] == "stream_error":
                        raise Exception(event["error"]["message"])
                    elif event["event_type"] == "text_generation":
                        self.add_text(event["text"])
                        yield event["text"]
                    elif event["event_type"] == "stream_end":
                        return

            if buffer:
                # If there's any remaining data in the buffer, process it
                event = json.loads(buffer)
                if event["event_type"] == "text_generation":
                    self.add_text(event["text"])
                    yield event["text"]



from __future__ import annotations
from typing import (
    List,
    Optional,
    Union,
    Literal,
    Dict,
    TYPE_CHECKING,
)

from ..models import RootModelList
from ..resource import APIResource
from .. import models
from .. import types
from ..util import remove_none_values, get_local_params, handle_comparison_params
from .video import Video

if TYPE_CHECKING:
    from ..client import TwelveLabs


class Index(APIResource):
    video: Video

    def __init__(self, client: TwelveLabs) -> None:
        super().__init__(client)
        self.video = Video(client)

    def retrieve(self, id: str, **kwargs) -> models.Index:
        res = self._get(f"indexes/{id}", **kwargs)
        return models.Index(self, **res)

    def list(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        model_options: Optional[Literal["visual", "audio"]] = None,
        model_family: Optional[Literal["marengo", "pegasus"]] = None,
        page: Optional[int] = 1,
        page_limit: Optional[int] = 10,
        sort_by: Optional[str] = "created_at",
        sort_option: Optional[str] = "desc",
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        **kwargs,
    ) -> RootModelList[models.Index]:
        params = {
            "_id": id,
            "index_name": name,
            "model_options": model_options,
            "model_family": model_family,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
            "created_at": created_at,
            "updated_at": updated_at,
        }
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)
        res = self._get("indexes", params=remove_none_values(params), **kwargs)

        return RootModelList([models.Index(self, **index) for index in res["data"]])

    def list_pagination(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        model_options: Optional[Literal["visual", "audio"]] = None,
        model_family: Optional[Literal["marengo", "pegasus"]] = None,
        page: Optional[int] = 1,
        page_limit: Optional[int] = 10,
        sort_by: Optional[str] = "created_at",
        sort_option: Optional[str] = "desc",
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        **kwargs,
    ) -> models.IndexListWithPagination:
        local_params = get_local_params(locals().items())
        params = {
            "_id": id,
            "index_name": name,
            "model_options": model_options,
            "model_family": model_family,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
            "created_at": created_at,
            "updated_at": updated_at,
        }
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)

        res = self._get("indexes", params=remove_none_values(params), **kwargs)

        data = [models.Index(self, **index) for index in res["data"]]
        page_info = models.PageInfo(**res["page_info"])

        return models.IndexListWithPagination(
            self,
            local_params,
            **{"data": data, "page_info": page_info},
        )

    def create(
        self,
        name: str,
        models: List[types.IndexModel],
        *,
        addons: Optional[List[str]] = None,
        **kwargs,
    ) -> models.Index:
        json = {
            "index_name": name,
            "models": list(
                map(
                    lambda model: {
                        "model_name": model["name"],
                        "model_options": model["options"],
                    },
                    models,
                )
            ),
            "addons": addons,
        }
        resp = self._post("indexes", json=json, **kwargs)
        return self.retrieve(resp["_id"])

    def update(self, id: str, name: str, **kwargs) -> None:
        json = {"index_name": name}
        self._put(f"indexes/{id}", json=json, **kwargs)

    def delete(self, id: str, **kwargs) -> None:
        self._delete(f"indexes/{id}", **kwargs)



from __future__ import annotations

from typing import Optional, TYPE_CHECKING, Dict, Any, List, Union, Literal
from pydantic import PrivateAttr

from ._base import ObjectWithTimestamp, BaseModel, ModelMixin, PageInfo, RootModelList

from .embed import CreateEmbeddingsResult
from .generate import (
    GenerateSummarizeResult,
    GenerateOpenEndedTextResult,
)

if TYPE_CHECKING:
    from ..resources import Video as VideoResource


class VideoMetadata(BaseModel):
    filename: str
    duration: float
    fps: float
    width: int
    height: int
    size: int

    class Config:
        extra = "allow"  # This allows extra fields


class VideoHLS(BaseModel):
    video_url: Optional[str] = None
    thumbnail_urls: Optional[List[str]] = None
    status: Optional[str] = None
    updated_at: Optional[str] = None


class VideoValue(BaseModel):
    start: float
    end: float
    value: str


class VideoSource(BaseModel):
    type: str
    name: Optional[str] = None
    url: Optional[str] = None


class Video(ObjectWithTimestamp):
    _resource: VideoResource = PrivateAttr()
    _index_id: str = PrivateAttr()
    system_metadata: VideoMetadata
    user_metadata: Optional[Dict[str, Any]] = None
    hls: Optional[VideoHLS] = None
    source: Optional[VideoSource] = None
    indexed_at: Optional[str] = None
    created_at: str
    updated_at: Optional[str] = None
    embedding: Optional[CreateEmbeddingsResult] = None

    def __init__(self, resource: VideoResource, index_id: str, **data):
        super().__init__(**data)
        self._resource = resource
        self._index_id = index_id

    # Video related methods

    def update(
        self,
        *,
        user_metadata: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> None:
        return self._resource._client.index.video.update(
            self._index_id, self.id, user_metadata=user_metadata, **kwargs
        )

    def delete(
        self,
        **kwargs,
    ) -> None:
        return self._resource._client.index.video.delete(
            self._index_id, self.id, **kwargs
        )

    # Generate related methods

    def generate_summarize(
        self,
        type: Literal["summary", "chapter", "highlight"],
        *,
        prompt: Optional[str] = None,
        **kwargs,
    ) -> GenerateSummarizeResult:
        return self._resource._client.generate.summarize(
            self.id, type, prompt=prompt, **kwargs
        )

    def generate_text(
        self,
        prompt: Optional[str] = None,
        **kwargs,
    ) -> GenerateOpenEndedTextResult:
        return self._resource._client.generate.text(
            self.id, type, prompt=prompt, **kwargs
        )


class VideoListWithPagination(ModelMixin, BaseModel):
    _resource: VideoResource = PrivateAttr()
    _origin_params: Dict[str, Any] = PrivateAttr()
    data: RootModelList[Video] = []
    page_info: PageInfo

    def __init__(self, resource: VideoResource, origin_params: Dict[str, Any], **data):
        super().__init__(**data)
        self._resource = resource
        self._origin_params = origin_params

    def __iter__(self):
        return self

    def __next__(self) -> RootModelList[Video]:
        if self.page_info.page >= self.page_info.total_page:
            raise StopIteration
        params = self._origin_params
        params["page"] = self.page_info.page + 1
        res = self._resource.list_pagination(**params)
        self.page_info = res.page_info
        return res.data



from .index import Index
from .task import Task
from .video import Video
from .search import Search
from .generate import Generate
from .embed import Embed, EmbedTask

__all__ = [
    "Index",
    "Task",
    "Video",
    "Search",
    "Generate",
    "Embed",
    "EmbedTask",
]



from __future__ import annotations
from io import BytesIO
from typing import Optional, Union, BinaryIO, List, Dict, Any, TYPE_CHECKING

from ..models import RootModelList
from ..resource import APIResource
from .. import models
from ..util import remove_none_values, get_local_params, handle_comparison_params

if TYPE_CHECKING:
    from ..client import TwelveLabs


class TaskTransfer(APIResource):
    def import_videos(
        self,
        integration_id: str,
        index_id: str,
        user_metadata: Optional[Dict[str, Any]] = None,
        incremental_import: Optional[bool] = None,
        retry_failed: Optional[bool] = None,
        **kwargs,
    ) -> models.TransferImportResponse:
        json = {
            "index_id": index_id,
            "user_metadata": user_metadata,
            "incremental_import": incremental_import,
            "retry_failed": retry_failed,
        }
        res = self._post(
            f"tasks/transfers/import/{integration_id}", json=json, **kwargs
        )
        return models.TransferImportResponse(**res)

    def import_status(
        self, integration_id: str, index_id: str, **kwargs
    ) -> models.TransferImportStatusResponse:
        params = {"index_id": index_id}
        res = self._get(
            f"tasks/transfers/import/{integration_id}/status", params=params, **kwargs
        )
        return models.TransferImportStatusResponse(**res)

    def import_logs(
        self, integration_id: str, **kwargs
    ) -> RootModelList[models.TransferImportLog]:
        res = self._get(f"tasks/transfers/import/{integration_id}/logs", **kwargs)
        return RootModelList([models.TransferImportLog(**log) for log in res["data"]])


class Task(APIResource):
    transfers: TaskTransfer

    def __init__(self, client: TwelveLabs) -> None:
        super().__init__(client)
        self.transfers = TaskTransfer(client)

    def retrieve(self, id: str, **kwargs) -> models.Task:
        res = self._get(f"tasks/{id}", **kwargs)
        return models.Task(self, **res)

    def list(
        self,
        *,
        index_id: Optional[str] = None,
        filename: Optional[str] = None,
        duration: Optional[float] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> RootModelList[models.Task]:
        params = {
            "index_id": index_id,
            "filename": filename,
            "duration": duration,
            "width": width,
            "height": height,
            "created_at": created_at,
            "updated_at": updated_at,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
        }
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)
        res = self._get("tasks", params=remove_none_values(params), **kwargs)
        return RootModelList([models.Task(self, **task) for task in res["data"]])

    def list_pagination(
        self,
        *,
        index_id: Optional[str] = None,
        filename: Optional[str] = None,
        duration: Optional[float] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> models.TaskListWithPagination:
        local_params = get_local_params(locals().items())
        params = {
            "index_id": index_id,
            "filename": filename,
            "duration": duration,
            "width": width,
            "height": height,
            "created_at": created_at,
            "updated_at": updated_at,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
        }
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)
        res = self._get("tasks", params=remove_none_values(params), **kwargs)

        data = [models.Task(self, **task) for task in res["data"]]
        page_info = models.PageInfo(**res["page_info"])

        return models.TaskListWithPagination(
            self,
            local_params,
            **{"data": data, "page_info": page_info},
        )

    def create(
        self,
        index_id: str,
        *,
        file: Union[str, BinaryIO, None] = None,
        url: Optional[str] = None,
        transcription_file: Union[str, BinaryIO, None] = None,
        transcription_url: Optional[str] = None,
        language: Optional[str] = None,
        enable_video_stream: Optional[bool] = None,
        **kwargs,
    ) -> models.Task:
        if not file and not url:
            raise ValueError("Either file or url must be provided")
        data = {
            "index_id": index_id,
            "video_url": url,
            "transcription_url": transcription_url,
            "language": language,
            "enable_video_stream": enable_video_stream,
        }

        files = {}
        opened_files: List[BinaryIO] = []

        if file is not None:
            if isinstance(file, str):
                file = open(file, "rb")
                opened_files.append(file)
            files["video_file"] = file

        if transcription_file is not None:
            if isinstance(transcription_file, str):
                transcription_file = open(transcription_file, "rb")
                opened_files.append(transcription_file)
            files["transcription_file"] = transcription_file
            data["provide_transcription"] = True
        if transcription_url is not None:
            data["provide_transcription"] = True

        if url is not None:
            files["video_url"] = BytesIO(url.encode())
        if transcription_url is not None:
            files["transcription_url"] = BytesIO(transcription_url.encode())

        try:
            res = self._post(
                "tasks", data=remove_none_values(data), files=files, **kwargs
            )
            return self.retrieve(res["_id"])
        finally:
            for file in opened_files:
                file.close()

    def create_bulk(
        self,
        index_id: str,
        *,
        files: Optional[List[Union[str, BinaryIO, None]]] = None,
        urls: Optional[List[str]] = None,
        language: Optional[str] = None,
        enable_video_stream: Optional[bool] = None,
        **kwargs,
    ) -> RootModelList[models.Task]:
        if not files and not urls:
            raise ValueError("Either files or urls must be provided")

        tasks = []
        if files:
            for file in files:
                try:
                    task = self.create(
                        index_id,
                        file=file,
                        language=language,
                        enable_video_stream=enable_video_stream,
                        **kwargs,
                    )
                    tasks.append(task)
                except Exception as e:
                    print(f"Error processing file {file}: {e}")
                    continue

        if urls:
            for url in urls:
                try:
                    task = self.create(
                        index_id,
                        url=url,
                        language=language,
                        enable_video_stream=enable_video_stream,
                        **kwargs,
                    )
                    tasks.append(task)
                except Exception as e:
                    print(f"Error processing url {url}: {e}")
                    continue

        return RootModelList(tasks)

    def delete(self, id: str, **kwargs) -> None:
        self._delete(f"tasks/{id}", **kwargs)

    def status(self, index_id: str, **kwargs) -> models.TaskStatus:
        params = {"index_id": index_id}
        res = self._get("tasks/status", params=params, **kwargs)
        return models.TaskStatus(**res)



from __future__ import annotations
from io import BytesIO
from typing import Union, List, Literal, Optional, BinaryIO, TYPE_CHECKING

from ..models._base import RootModelList
from ..resource import APIResource
from .. import models
from ..util import remove_none_values, get_local_params


if TYPE_CHECKING:
    from ..client import TwelveLabs


class EmbedTask(APIResource):
    def retrieve(self, id: str, **kwargs) -> models.EmbeddingsTask:
        res = self._get(f"embed/tasks/{id}", **kwargs)
        return models.EmbeddingsTask(self, **res)

    def list(
        self,
        *,
        started_at: Optional[str] = None,
        ended_at: Optional[str] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        status: Optional[Literal["processing", "ready", "failed"]] = None,
        **kwargs,
    ) -> RootModelList[models.EmbeddingsTask]:
        params = {
            "started_at": started_at,
            "ended_at": ended_at,
            "page": page,
            "page_limit": page_limit,
            "status": status,
        }
        res = self._get("embed/tasks", params=remove_none_values(params), **kwargs)
        return RootModelList(
            [models.EmbeddingsTask(self, **task) for task in res["data"]]
        )

    def list_pagination(
        self,
        *,
        started_at: Optional[str] = None,
        ended_at: Optional[str] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        status: Optional[Literal["processing", "ready", "failed"]] = None,
        **kwargs,
    ) -> models.EmbeddingsTaskListWithPagination:
        local_params = get_local_params(locals().items())
        params = {
            "started_at": started_at,
            "ended_at": ended_at,
            "page": page,
            "page_limit": page_limit,
            "status": status,
        }
        res = self._get("embed/tasks", params=remove_none_values(params), **kwargs)

        data = [models.EmbeddingsTask(self, **task) for task in res["data"]]
        page_info = models.PageInfo(**res["page_info"])

        return models.EmbeddingsTaskListWithPagination(
            self,
            local_params,
            **{"data": data, "page_info": page_info},
        )

    def create(
        self,
        model_name: Literal["Marengo-retrieval-2.7"],
        *,
        video_file: Union[str, BinaryIO, None] = None,
        video_url: Optional[str] = None,
        video_start_offset_sec: Optional[float] = None,
        video_end_offset_sec: Optional[float] = None,
        video_clip_length: Optional[int] = None,
        video_embedding_scopes: Optional[List[Literal["clip", "video"]]] = None,
        **kwargs,
    ) -> models.EmbeddingsTask:
        if not video_file and not video_url:
            raise ValueError("Either video_file or video_url must be provided")
        data = {
            "model_name": model_name,
            "video_embedding_scope": video_embedding_scopes,
            "video_url": video_url,
            "video_start_offset_sec": video_start_offset_sec,
            "video_end_offset_sec": video_end_offset_sec,
            "video_clip_length": video_clip_length,
        }

        files = {}
        opened_files: List[BinaryIO] = []
        if video_file is not None:
            if isinstance(video_file, str):
                video_file = open(video_file, "rb")
                opened_files.append(video_file)
            files["video_file"] = video_file
        if video_url is not None:
            files["video_url"] = BytesIO(video_url.encode())

        try:
            res = self._post(
                "embed/tasks",
                data=remove_none_values(data),
                files=files,
                **kwargs,
            )
            task_id = res["_id"]
            return self.retrieve(task_id)
        finally:
            for file in opened_files:
                file.close()

    def create_bulk(
        self,
        model_name: Literal["Marengo-retrieval-2.7"],
        videos: List[models.CreateEmbeddingsTaskVideoParams],
        **kwargs,
    ) -> RootModelList[models.EmbeddingsTask]:
        tasks = []
        for video_params in videos:
            try:
                task = self.create(
                    model_name,
                    video_file=video_params.file,
                    video_url=video_params.url,
                    video_start_offset_sec=video_params.start_offset_sec,
                    video_end_offset_sec=video_params.end_offset_sec,
                    video_clip_length=video_params.clip_length,
                    scopes=video_params.scopes,
                    **kwargs,
                )
                tasks.append(task)
            except Exception as e:
                print(f"Error creating task with video: {e}")
                continue
        return tasks

    def status(self, task_id: str, **kwargs) -> models.EmbeddingsTaskStatus:
        res = self._get(f"embed/tasks/{task_id}/status", **kwargs)
        return models.EmbeddingsTaskStatus(**res)


class Embed(APIResource):
    task: EmbedTask

    def __init__(self, client: TwelveLabs) -> None:
        super().__init__(client)
        self.task = EmbedTask(client)

    def create(
        self,
        model_name: Literal["Marengo-retrieval-2.7"],
        *,
        # text params
        text: str = None,
        text_truncate: Literal["none", "start", "end"] = None,
        # audio params
        audio_url: str = None,
        audio_file: Union[str, BinaryIO, None] = None,
        audio_start_offset_sec: Optional[float] = None,
        # image params
        image_url: str = None,
        image_file: Union[str, BinaryIO, None] = None,
        **kwargs,
    ) -> models.CreateEmbeddingsResult:
        if not any([text, audio_url, audio_file, image_url, image_file]):
            raise ValueError(
                "At least one of text, audio_url, audio_file, image_url, image_file must be provided"
            )
        data = {
            "model_name": model_name,
            "text": text,
            "text_truncate": text_truncate,
            "audio_url": audio_url,
            "audio_start_offset_sec": audio_start_offset_sec,
            "image_url": image_url,
        }
        files = {}
        opened_files: List[BinaryIO] = []

        if text is not None:
            files["text"] = BytesIO(text.encode())
        if audio_url is not None:
            files["audio_url"] = BytesIO(audio_url.encode())
        if image_url is not None:
            files["image_url"] = BytesIO(image_url.encode())

        if audio_file is not None:
            if isinstance(audio_file, str):
                audio_file = open(audio_file, "rb")
                opened_files.append(audio_file)
            files["audio_file"] = audio_file
        if image_file is not None:
            if isinstance(image_file, str):
                image_file = open(image_file, "rb")
                opened_files.append(image_file)
            files["image_file"] = image_file

        try:
            res = self._post(
                "embed",
                data=remove_none_values(data),
                files=files,
                **kwargs,
            )
            return models.CreateEmbeddingsResult(**res)
        finally:
            for file in opened_files:
                file.close()



from __future__ import annotations

from typing import List, Optional, TYPE_CHECKING, Union, BinaryIO, Literal, Dict, Any
from pydantic import Field, PrivateAttr

from ._base import ModelMixin, BaseModel, ObjectWithTimestamp, PageInfo, RootModelList

if TYPE_CHECKING:
    from ..resources import Index as IndexResource
    from . import Task, TaskStatus, SearchResult, Video, VideoListWithPagination


class Model(BaseModel):
    name: str = Field(alias="model_name")
    # conversation, text_in_video, and logo are to keep backward compatibility with the old models
    options: List[
        Literal["visual", "audio", "conversation", "text_in_video", "logo"]
    ] = Field(alias="model_options")
    addons: Optional[List[str]] = None
    finetuned: Optional[bool] = None

    class Config:
        # Disable the protected namespace restriction for options
        protected_namespaces = ()


class Index(ObjectWithTimestamp):
    _resource: IndexResource = PrivateAttr()
    name: str = Field(alias="index_name")
    models: RootModelList[Model]
    video_count: int
    total_duration: float
    expires_at: Optional[str] = None

    def __init__(self, resource: IndexResource, **data):
        super().__init__(**data)
        self._resource = resource

    # Index related methods

    def retrieve(self, **kwargs) -> Index:
        return self._resource.retrieve(self.id, **kwargs)

    def update(self, name: str, **kwargs) -> None:
        return self._resource.update(self.id, name, **kwargs)

    def delete(self, **kwargs) -> None:
        return self._resource.delete(self.id, **kwargs)

    # Task related methods

    def create_task(
        self,
        *,
        file: Union[str, BinaryIO, None] = None,
        url: Optional[str] = None,
        transcription_file: Union[str, BinaryIO, None] = None,
        transcription_url: Optional[str] = None,
        language: Optional[str] = None,
        **kwargs,
    ) -> Task:
        return self._resource._client.task.create(
            self.id,
            file=file,
            url=url,
            transcript_file=transcription_file,
            transcription_url=transcription_url,
            language=language,
            **kwargs,
        )

    def task_status(self, **kwargs) -> TaskStatus:
        return self._resource._client.task.status(self.id, **kwargs)

    # Video related methods
    def list_video(
        self,
        *,
        filename: Optional[str] = None,
        size: Optional[Union[int, Dict[str, int]]] = None,
        width: Optional[Union[int, Dict[str, int]]] = None,
        height: Optional[Union[int, Dict[str, int]]] = None,
        duration: Optional[Union[int, Dict[str, int]]] = None,
        fps: Optional[Union[int, Dict[str, int]]] = None,
        user_metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        indexed_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> RootModelList[Video]:
        return self._resource.video.list(
            self.id,
            filename=filename,
            size=size,
            width=width,
            height=height,
            duration=duration,
            fps=fps,
            user_metadata=user_metadata,
            created_at=created_at,
            updated_at=updated_at,
            indexed_at=indexed_at,
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            **kwargs,
        )

    def list_video_pagination(
        self,
        *,
        filename: Optional[str] = None,
        size: Optional[Union[int, Dict[str, int]]] = None,
        width: Optional[Union[int, Dict[str, int]]] = None,
        height: Optional[Union[int, Dict[str, int]]] = None,
        duration: Optional[Union[int, Dict[str, int]]] = None,
        fps: Optional[Union[int, Dict[str, int]]] = None,
        user_metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        indexed_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> VideoListWithPagination:
        return self._resource.video.list_pagination(
            self.id,
            filename=filename,
            size=size,
            width=width,
            height=height,
            duration=duration,
            fps=fps,
            user_metadata=user_metadata,
            created_at=created_at,
            updated_at=updated_at,
            indexed_at=indexed_at,
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            **kwargs,
        )

    # Search related methods

    def query(
        self,
        options: Literal["visual", "audio"],
        *,
        query_text: str = None,
        query_media_type: Literal["image"] = None,
        query_media_file: Union[str, BinaryIO, None] = None,
        query_media_url: str = None,
        group_by: Optional[Literal["video", "clip"]] = None,
        threshold: Optional[Literal["high", "medium", "low", "none"]] = None,
        operator: Optional[Literal["or", "and"]] = None,
        filter: Optional[Dict[str, Any]] = None,
        page_limit: Optional[int] = None,
        sort_option: Optional[Literal["score", "clip_count"]] = None,
        adjust_confidence_level: Optional[float] = None,
        **kwargs,
    ) -> SearchResult:
        return self._resource._client.search.query(
            self.id,
            options,
            query_text=query_text,
            query_media_type=query_media_type,
            query_media_file=query_media_file,
            query_media_url=query_media_url,
            group_by=group_by,
            threshold=threshold,
            operator=operator,
            filter=filter,
            page_limit=page_limit,
            sort_option=sort_option,
            adjust_confidence_level=adjust_confidence_level,
            **kwargs,
        )


class IndexListWithPagination(ModelMixin, BaseModel):
    _resource: IndexResource = PrivateAttr()
    _origin_params: Dict[str, Any] = PrivateAttr()
    data: List[Index] = []
    page_info: PageInfo

    def __init__(self, resource: IndexResource, origin_params: Dict[str, Any], **data):
        super().__init__(**data)
        self._resource = resource
        self._origin_params = origin_params

    def __iter__(self):
        return self

    def __next__(self) -> List[Index]:
        if self.page_info.page >= self.page_info.total_page:
            raise StopIteration
        params = self._origin_params
        params["page"] = self.page_info.page + 1
        res = self._resource.list_pagination(**params)
        self.page_info = res.page_info
        return res.data



from __future__ import annotations

from typing import (
    List,
    Union,
    BinaryIO,
    Optional,
    Literal,
    Callable,
    Any,
    TYPE_CHECKING,
)
from pydantic import PrivateAttr, Field

from ._base import BaseModel, Object, RootModelList, PageInfo

if TYPE_CHECKING:
    from ..resources import EmbedTask as EmbedTaskResource


class CreateEmbeddingsTaskVideoParams:
    file: Union[str, BinaryIO, None]
    url: Optional[str]
    start_offset_sec: Optional[float]
    end_offset_sec: Optional[float]
    clip_length: Optional[int]
    scopes: Optional[List[Literal["clip", "video"]]]

    def __init__(
        self,
        file: Union[str, BinaryIO, None] = None,
        *,
        url: Optional[str] = None,
        start_offset_sec: Optional[float] = None,
        end_offset_sec: Optional[float] = None,
        clip_length: Optional[int] = None,
        scopes: Optional[List[Literal["clip", "video"]]] = None,
    ):
        self.file = file
        self.url = url
        self.start_offset_sec = start_offset_sec
        self.end_offset_sec = end_offset_sec
        self.clip_length = clip_length
        self.scopes = scopes


class EmbeddingMediaMetadata(BaseModel):
    input_url: Optional[str] = None
    input_filename: Optional[str] = None


class Embedding(BaseModel):
    segments: Optional[List[SegmentEmbedding]] = None
    error_message: Optional[str] = None
    metadata: Optional[EmbeddingMediaMetadata] = None


class SegmentEmbedding(BaseModel):
    start_offset_sec: Optional[float] = None
    end_offset_sec: Optional[float] = None
    embedding_scope: Optional[str] = None
    embeddings_float: Optional[List[float]] = Field(default=None, alias="float")


class CreateEmbeddingsResult(BaseModel):
    model_name: str
    text_embedding: Optional[Embedding] = None
    image_embedding: Optional[Embedding] = None
    video_embedding: Optional[Embedding] = None
    audio_embedding: Optional[Embedding] = None

    class Config:
        # Disable the protected namespace restriction for model_name
        protected_namespaces = ()


class EmbeddingMetadata(BaseModel):
    input_url: Optional[str] = None
    input_filename: Optional[str] = None
    video_clip_length: Optional[int] = None
    video_embedding_scope: Optional[List[str]] = None
    duration: Optional[float] = None


class EmbeddingsTaskStatus(Object):
    model_name: str
    status: str
    metadata: Optional[EmbeddingMetadata] = None
    video_embedding: Optional[Embedding] = None

    class Config:
        # Disable the protected namespace restriction for model_name
        protected_namespaces = ()


class EmbeddingsTask(Object):
    _resource: EmbedTaskResource = PrivateAttr()
    model_name: str
    status: str
    video_embedding: Optional[Embedding] = None
    created_at: Optional[str] = None

    class Config:
        # Disable the protected namespace restriction for model_name
        protected_namespaces = ()

    def __init__(self, resource: EmbedTaskResource, **data):
        super().__init__(**data)
        self._resource = resource

    @property
    def done(self) -> bool:
        return self.status in ("ready", "failed")

    def retrieve(self, **kwargs) -> EmbeddingsTask:
        return self._resource.retrieve(self.id, **kwargs)

    def update_status(self, **kwargs) -> str:
        res = self._resource.status(self.id, **kwargs)
        self.status = res.status
        return self.status

    def wait_for_done(
        self,
        *,
        sleep_interval: float = 5.0,
        callback: Optional[Callable[[EmbeddingsTask], None]] = None,
        **kwargs,
    ) -> str:
        if sleep_interval <= 0:
            raise ValueError("sleep_interval must be greater than 0")
        while not self.done:
            self._resource._sleep(sleep_interval)
            try:
                self.update_status(**kwargs)
            except Exception as e:
                print(f"Retrieving status failed: {e}. Retrying..")
                continue
            if callback is not None:
                callback(self)
        return self.status


class EmbeddingsTaskListWithPagination(BaseModel):
    _resource: EmbedTaskResource = PrivateAttr()
    _origin_params: dict[str, Any] = PrivateAttr()
    data: RootModelList[EmbeddingsTask] = []
    page_info: PageInfo

    def __init__(
        self, resource: EmbedTaskResource, origin_params: dict[str, Any], **data
    ):
        super().__init__(**data)
        self._resource = resource
        self._origin_params = origin_params

    def __iter__(self):
        return self

    def __next__(self) -> RootModelList[EmbeddingsTask]:
        if self.page_info.page >= self.page_info.total_page:
            raise StopIteration
        params = self._origin_params
        params["page"] = self.page_info.page + 1
        res = self._resource.list_pagination(**params)
        self.page_info = res.page_info
        return res.data



from typing import Optional, Dict, Any, Union

from ..models._base import RootModelList
from ..resource import APIResource
from .. import models
from ..util import (
    remove_none_values,
    get_local_params,
    handle_comparison_params,
)


class Video(APIResource):
    def retrieve(
        self,
        index_id: str,
        id: str,
        *,
        embed: Optional[bool] = None,
        **kwargs,
    ) -> models.Video:
        params = {
            "embed": embed,
        }
        res = self._get(
            f"indexes/{index_id}/videos/{id}",
            params=remove_none_values(params),
            **kwargs,
        )
        return models.Video(self, index_id, **res)

    def list(
        self,
        index_id: str,
        *,
        filename: Optional[str] = None,
        size: Optional[Union[int, Dict[str, int]]] = None,
        width: Optional[Union[int, Dict[str, int]]] = None,
        height: Optional[Union[int, Dict[str, int]]] = None,
        duration: Optional[Union[int, Dict[str, int]]] = None,
        fps: Optional[Union[int, Dict[str, int]]] = None,
        user_metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        indexed_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> RootModelList[models.Video]:
        params = {
            "filename": filename,
            "size": size,
            "width": width,
            "height": height,
            "duration": duration,
            "fps": fps,
            "user_metadata": user_metadata,
            "created_at": created_at,
            "updated_at": updated_at,
            "indexed_at": indexed_at,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
        }
        handle_comparison_params(params, "fps", fps)
        handle_comparison_params(params, "width", width)
        handle_comparison_params(params, "height", height)
        handle_comparison_params(params, "size", size)
        handle_comparison_params(params, "duration", duration)
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)
        handle_comparison_params(params, "indexed_at", indexed_at)
        res = self._get(
            f"indexes/{index_id}/videos", params=remove_none_values(params), **kwargs
        )
        return RootModelList(
            [models.Video(self, index_id, **video) for video in res["data"]]
        )

    def list_pagination(
        self,
        index_id: str,
        *,
        filename: Optional[str] = None,
        size: Optional[Union[int, Dict[str, int]]] = None,
        width: Optional[Union[int, Dict[str, int]]] = None,
        height: Optional[Union[int, Dict[str, int]]] = None,
        duration: Optional[Union[int, Dict[str, int]]] = None,
        fps: Optional[Union[int, Dict[str, int]]] = None,
        user_metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[Union[str, Dict[str, str]]] = None,
        updated_at: Optional[Union[str, Dict[str, str]]] = None,
        indexed_at: Optional[Union[str, Dict[str, str]]] = None,
        page: Optional[int] = None,
        page_limit: Optional[int] = None,
        sort_by: Optional[str] = None,
        sort_option: Optional[str] = None,
        **kwargs,
    ) -> models.VideoListWithPagination:
        local_params = get_local_params(locals().items())
        params = {
            "filename": filename,
            "size": size,
            "width": width,
            "height": height,
            "duration": duration,
            "fps": fps,
            "user_metadata": user_metadata,
            "created_at": created_at,
            "updated_at": updated_at,
            "indexed_at": indexed_at,
            "page": page,
            "page_limit": page_limit,
            "sort_by": sort_by,
            "sort_option": sort_option,
        }
        handle_comparison_params(params, "fps", fps)
        handle_comparison_params(params, "width", width)
        handle_comparison_params(params, "height", height)
        handle_comparison_params(params, "size", size)
        handle_comparison_params(params, "duration", duration)
        handle_comparison_params(params, "created_at", created_at)
        handle_comparison_params(params, "updated_at", updated_at)
        handle_comparison_params(params, "indexed_at", indexed_at)

        res = self._get(
            f"indexes/{index_id}/videos", params=remove_none_values(params), **kwargs
        )

        data = [models.Video(self, index_id, **video) for video in res["data"]]
        page_info = models.PageInfo(**res["page_info"])

        return models.VideoListWithPagination(
            self,
            local_params,
            **{"data": data, "page_info": page_info},
        )

    def update(
        self,
        index_id: str,
        id: str,
        *,
        user_metadata: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> None:
        json = {
            "user_metadata": user_metadata,
        }
        self._put(
            f"indexes/{index_id}/videos/{id}", json=remove_none_values(json), **kwargs
        )

    def delete(self, index_id: str, id: str, **kwargs) -> None:
        self._delete(f"indexes/{index_id}/videos/{id}", **kwargs)



import os
from typing import Union, Literal

from .constants import BASE_URL, LATEST_API_VERSION
from .base_client import APIClient
from . import resources


class TwelveLabs(APIClient):
    index: resources.Index
    task: resources.Task
    search: resources.Search
    generate: resources.Generate
    embed: resources.Embed

    base_url: str
    api_key: str

    def __init__(
        self,
        api_key: str,
        version: Union[
            str,
            Literal["v1.1", "v1.2", "v1.3"],
        ] = LATEST_API_VERSION,
    ) -> None:
        if version != LATEST_API_VERSION:
            print(
                f"[Warning] You manually set the API version to {version}, but this SDK version is not fully compatible with current API version, please use version 0.3.x or earlier"
            )
        assert (
            api_key,
            "Provide `api_key` to initialize a client. You can see the API Key in the Dashboard page: https://dashboard.playground.io",
        )
        base_url = f"{BASE_URL}/{version}/"
        custom_base_url = os.environ.get("TWELVELABS_BASE_URL")
        if custom_base_url is not None:
            base_url = f"{custom_base_url}/{version}/"

        self.base_url = base_url
        self.api_key = api_key
        super().__init__(base_url, api_key)

        self.index = resources.Index(self)
        self.task = resources.Task(self)
        self.search = resources.Search(self)
        self.generate = resources.Generate(self)
        self.embed = resources.Embed(self)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, tb):
        pass



from typing import Dict, Union, Any


def remove_none_values(data: Union[list, dict]):
    if isinstance(data, dict):
        return {key: value for key, value in data.items() if value is not None}
    elif isinstance(data, list):
        return [remove_none_values(item) for item in data]
    else:
        return data


def get_local_params(local_items: Dict[str, Any]):
    return {k: v for k, v in local_items if k not in ["self", "kwargs"]}


def get_data_with_default(dictionary: Dict, key: str, default=[]):
    return dictionary.get(key, default) if dictionary.get(key) is not None else default


def handle_comparison_params(params: Dict, key: str, value: Any):
    if isinstance(value, dict):
        for op, date in value.items():
            param_name = f"{key}[{op}]"
            params[param_name] = date
        params[key] = None
    else:
        params[key] = value
